-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftCastBarText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Cast Bar Text"] = true,
	["Cast bar text options"] = true,
	["Enable"] = true,
	["Show cast bar text"] = true,
	["Font"] = true, 
	["Sets the font for cast bar text"] = true,
	["Font Size"] = true,
	["Sets the font height of the cast bar text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on cast bar text"] = true,
	["Alignment"] = true,
	["Sets the alignment of the cast bar text"] = true,
	["Outline"] = true,
	["Sets the font outline for cast bar text"] = true,
	["Position"] = true,
	["Adjust cast bar text position"] = true,
	["Left Offset"] = true,
	["Sets the offset of the left of the text"] = true,
	["Right Offset"] = true,
	["Sets the offset of the right of the text"] = true,
	["Vertical Offset"] = true,
	["Sets the vertical offset of the text"] = true,
	["Color"] = true,
	["Sets the cast bar text color"] = true,
} end)

-----------------------------------------------------------------------------

local castBarTextFrame = nil

-----------------------------------------------------------------------------

AloftCastBarText = Aloft:NewModule("CastBarText", "AceEvent-2.0")
AloftCastBarText.db = Aloft:AcquireDBNamespace("castBarText")
Aloft:RegisterDefaults("castBarText", "profile", {
	visible			= true,
	font			= "Arial Narrow",
	fontSize		= 9,
	shadow			= true,
	alignment		= "LEFT",
	outline			= "None",
	offsets =
	{
		left		= 0,
		right		= 0,
		vertical	= 0,
	},
	color			= { 1, 1, 1, 1 }
})

Aloft.OnMenuRequest.args.castBarText =
{
	type = 'group',
	name = L["Cast Bar Text"],
	desc = L["Cast bar text options"],
	args =
	{
		visible =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show cast bar text"],
			get = function() return AloftCastBarText.db.profile.visible end,
			set = function(v) AloftCastBarText.db.profile.visible = v AloftCastBarText:Update() end
		},
		font =
		{
			type = "text", 
			name = L["Font"], 
			desc = L["Sets the font for cast bar text"],
			get = function() return AloftCastBarText.db.profile.font end,
			set = function(v) AloftCastBarText.db.profile.font = v AloftCastBarText:Update() end,
			validate = SML_font,
		},
		fontSize =
		{
			type = 'range',
			name = L["Font Size"],
			desc = L["Sets the font height of the cast bar text"],
			max = 16,
			min = 5,
			step = 1,
			get = function() return AloftCastBarText.db.profile.fontSize end,
			set = function(value) AloftCastBarText.db.profile.fontSize = value AloftCastBarText:Update() end
		},
		shadow =
		{
			type = 'toggle',
			name = L["Font Shadow"],
			desc = L["Show font shadow on cast bar text"],
			get = function() return AloftCastBarText.db.profile.shadow end,
			set = function(v) AloftCastBarText.db.profile.shadow = v AloftCastBarText:Update() end
		},
		alignment =
		{
			type = 'text',
			name = L["Alignment"],
			desc = L["Sets the alignment of the cast bar text"],
			get = function() return AloftCastBarText.db.profile.alignment end,
			set = function(value) AloftCastBarText.db.profile.alignment = value AloftCastBarText:Update() end,
			validate = {"LEFT", "CENTER", "RIGHT" },
		},
		outline =
		{
			type = 'text',
			name = L["Outline"],
			desc = L["Sets the font outline for cast bar text"],
			get = function() return AloftCastBarText.db.profile.outline end,
			set = function(value) AloftCastBarText.db.profile.outline = value AloftCastBarText:Update() end,
			validate = {"None", "Normal", "Thick" },
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust cast bar text position"],
			args =
			{
				left =
				{
					type = 'range',
					name = L["Left Offset"],
					desc = L["Sets the offset of the left of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarText.db.profile.offsets.left end,
					set = function(v) AloftCastBarText.db.profile.offsets.left = v AloftCastBarText:Update() end
				},
				right =
				{
					type = 'range',
					name = L["Right Offset"],
					desc = L["Sets the offset of the right of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarText.db.profile.offsets.right end,
					set = function(v) AloftCastBarText.db.profile.offsets.right = v AloftCastBarText:Update() end
				},
				vertical =
				{
					type = 'range',
					name = L["Vertical Offset"],
					desc = L["Sets the vertical offset of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarText.db.profile.offsets.vertical end,
					set = function(v) AloftCastBarText.db.profile.offsets.vertical = v AloftCastBarText:Update() end
				},
			},
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the cast bar text color"],
			get = function() return unpack(AloftCastBarText.db.profile.color) end,
			set = function(r, g, b, a) AloftCastBarText.db.profile.color = { r, g, b, a } AloftCastBarText:Update() end,
			hasAlpha = true
		},
	},
}

-----------------------------------------------------------------------------

function AloftCastBarText:Update()
	self:CreateFrame()
	for frame in Aloft:IterateVisibleNameplates() do
		self:SetupFrame(frame)
	end
end

-----------------------------------------------------------------------------

function AloftCastBarText:CreateFrame()
	if not self.db.profile.visible then
		if castBarTextFrame then
			self:UnregisterAllEvents()
			castBarTextFrame:Hide()
		end
		return
	end

	if not castBarTextFrame then
		castBarTextFrame = UIParent:CreateFontString()
	end
	
	self:PrepareText(castBarTextFrame, self.db.profile)

	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnCastBarValueChanged", "UpdateText")
	self:RegisterEvent("Aloft:OnCastBarShow", "OnCastBarShow")
	self:RegisterEvent("Aloft:OnCastBarHide", "OnCastBarHide")
end

-----------------------------------------------------------------------------

function AloftCastBarText:GetTargetSpellName()
	local _, _, displayName = UnitCastingInfo("target")
	if displayName then
		return displayName
	end
	_, _, displayName = UnitChannelInfo("target")
	if displayName then
		return displayName
	end
	return ""
end

-----------------------------------------------------------------------------

function AloftCastBarText:OnEnable()
	self:CreateFrame()
end

function AloftCastBarText:SetupFrame(frame)
	if castBarTextFrame and frame.aloftData.name == UnitName("target") then
		local castBar = frame.aloftData.castBar
		castBarTextFrame:ClearAllPoints()
		castBarTextFrame:SetPoint("TOPLEFT", castBar, "TOPLEFT", self.db.profile.offsets.left, self.db.profile.offsets.vertical)
		castBarTextFrame:SetPoint("BOTTOMRIGHT", castBar, "BOTTOMRIGHT", self.db.profile.offsets.right, self.db.profile.offsets.vertical)
	end	
end

function AloftCastBarText:UpdateText(frame)
	local castBar = frame.aloftData.castBar
	
	if castBarTextFrame and frame.aloftData.castBarStartTime ~= castBar:GetMinMaxValues() then
		castBarTextFrame:SetText(self:GetTargetSpellName())
	end
end

function AloftCastBarText:OnCastBarShow(frame)
	local profile = self.db.profile

	if castBarTextFrame and profile.visible then
		local castBar = frame.aloftData.castBar

		castBarTextFrame:SetText(self:GetTargetSpellName())
		castBarTextFrame:ClearAllPoints()
		castBarTextFrame:SetPoint("TOPLEFT", castBar, "TOPLEFT", profile.offsets.left, profile.offsets.vertical)
		castBarTextFrame:SetPoint("BOTTOMRIGHT", castBar, "BOTTOMRIGHT", profile.offsets.right, profile.offsets.vertical)
		castBarTextFrame:SetAlpha(1.0)
		castBarTextFrame:Show()

		frame.aloftData.castBarStartTime = castBar:GetMinMaxValues()

		self:ScheduleRepeatingEvent("AloftCastBarText:UpdateAlpha", self.UpdateAlpha, 0, self, select(3, frame:GetRegions()))
	end
end

function AloftCastBarText:OnCastBarHide(frame)
	self:CancelScheduledEvent("AloftCastBarText:UpdateAlpha")
	if castBarTextFrame then
		castBarTextFrame:Hide()
	end
end

function AloftCastBarText:UpdateAlpha(spellIconFrame)
	castBarTextFrame:SetAlpha(spellIconFrame:GetAlpha())
end

-----------------------------------------------------------------------------
